use palette::Oklch;
use serde::Serialize;
use wasm_bindgen::prelude::wasm_bindgen;

use super::{ColorDifference, Differ};

#[derive(Debug, Clone, Copy, Serialize)]
#[wasm_bindgen]
pub struct OklchDifference {
    pub hue: Differ,
    pub chroma: Differ,
    pub lightness: Differ,
}

impl ColorDifference for Oklch {
    type Difference = OklchDifference;

    fn difference(&self, other: &Self) -> Self::Difference {
        let hue = other.hue.into_positive_degrees() - self.hue.into_positive_degrees();
        let chroma = other.chroma - self.chroma;
        let lightness = other.l - self.l;

        OklchDifference {
            hue: Differ {
                delta: hue,
                percent: hue / self.hue.into_positive_degrees(),
            },
            chroma: Differ {
                delta: chroma,
                percent: chroma / self.chroma,
            },
            lightness: Differ {
                delta: lightness,
                percent: lightness / self.l,
            },
        }
    }

    fn difference_relative(&self, other: &Self) -> Self::Difference {
        let hue = if self.hue.into_positive_degrees() == 0.0 {
            0.0
        } else {
            (other.hue.into_positive_degrees() - self.hue.into_positive_degrees())
                / (1.0 - self.hue.into_positive_degrees())
        };
        let chroma = if self.chroma == 0.0 {
            0.0
        } else {
            (other.chroma - self.chroma) / self.chroma
        };
        let lightness = if self.l == 0.0 {
            0.0
        } else {
            (other.l - self.l) / (1.0 - self.l)
        };

        OklchDifference {
            hue: Differ {
                delta: other.hue.into_positive_degrees() - self.hue.into_positive_degrees(),
                percent: hue,
            },
            chroma: Differ {
                delta: other.chroma - self.chroma,
                percent: chroma,
            },
            lightness: Differ {
                delta: other.l - self.l,
                percent: lightness,
            },
        }
    }
}
