import { isEqual } from 'lodash-es';
import { useState } from 'react';
import { useCopyColor } from '../hooks/useCopyColor';
import { ColorComponentInput } from './ColorComponentInput';
import styles from './ColorPicker.module.css';
import { HSegmentedControl } from './HSegmentedControl';
import { HslAssemble } from './HslAsssemble';
import { RGBAssemble } from './RGBAssemble';

type ColorPickerProps = {
  color?: string | null;
  onSelect?: (color: string) => void;
};

export function ColorPicker({ color, onSelect }: ColorPickerProps) {
  const [pickMode, setMode] = useState<'rgb' | 'hsl' | 'input'>('rgb');
  const [selectedColor, setSelectedColor] = useState<string>(color ?? '000000');
  const handleColorSelect = (color: string) => {
    setSelectedColor(color);
    onSelect?.(color);
  };
  const copyColor = useCopyColor();

  return (
    <div className={styles.color_picker}>
      <div className={styles.color_preview} style={{ backgroundColor: `#${selectedColor}` }} />
      <div className={styles.color_code} onClick={() => copyColor(selectedColor)}>
        #{selectedColor}
      </div>
      <HSegmentedControl
        options={[
          { label: 'RGB', value: 'rgb' },
          { label: 'HSL', value: 'hsl' },
          { label: 'Input', value: 'input' },
        ]}
        value={pickMode}
        onChange={(value) => setMode(value as 'rgb' | 'hsl' | 'input')}
      />
      {isEqual(pickMode, 'rgb') && (
        <RGBAssemble color={selectedColor} onChange={handleColorSelect} />
      )}
      {isEqual(pickMode, 'hsl') && (
        <HslAssemble color={selectedColor} onChange={handleColorSelect} />
      )}
      {isEqual(pickMode, 'input') && (
        <ColorComponentInput color={selectedColor} onChange={handleColorSelect} />
      )}
    </div>
  );
}
