import cx from 'clsx';
import { isEqual, isNil } from 'lodash-es';
import { useCallback, useEffect, useState } from 'react';
import styles from './Tab.module.css';

type TabProps = {
  tabs: { title: string; id: unknown }[];
  activeTab?: unknown;
  onActive?: (id: unknown) => void;
};

export function Tab({ tabs = [], activeTab, onActive }: TabProps) {
  const [active, setActive] = useState(() =>
    isNil(activeTab) ? 0 : tabs.findIndex((tab) => isEqual(tab.id, activeTab)),
  );
  const handleActivate = useCallback((index: number) => {
    setActive(index);
    onActive?.(tabs[index].id);
  }, []);

  useEffect(() => {
    const activeIndex = tabs.findIndex((tab) => isEqual(tab.id, activeTab));
    if (!isNil(activeIndex) && !isEqual(activeIndex, -1) && !isEqual(activeIndex, active)) {
      setActive(activeIndex);
    }
  }, [activeTab]);

  return (
    <div className={styles.tabs_container}>
      {tabs.map((tab, index) => (
        <div
          key={`tab_${index}_${tab.id}`}
          className={cx(styles.tab, isEqual(index, active) && styles.actived)}
          onClick={() => handleActivate(index)}>
          {tab.title}
        </div>
      ))}
    </div>
  );
}
