import cx from 'clsx';
import { isEqual } from 'lodash-es';
import { useMemo } from 'react';
import { useColorFunction } from '../../ColorFunctionContext';
import { useCopyColor } from '../../hooks/useCopyColor';
import styles from './ColorColumn.module.css';

type ColorBlockProps = {
  dimmed: boolean;
  color: string;
  isRoot?: boolean;
};

function ColorBlock({ dimmed, color, isRoot = false }: ColorBlockProps) {
  const copyColor = useCopyColor();

  return (
    <div
      className={styles.color_block}
      onClick={() => copyColor(color.startsWith('#') ? color : `#${color}`)}>
      <div className={styles.bg} style={{ backgroundColor: `#${color}` }} />
      <div className={cx(styles.dim_overlay, dimmed && styles.show_overlay)} />
      <div className={cx(isRoot && styles.active)} />
    </div>
  );
}

type ColorWheelProps = {
  actived: boolean;
  rotate: number;
  rootColor: string;
  tones: number;
  step: number;
};

export function ColorColumn({ actived, rotate, rootColor, tones, step }: ColorWheelProps) {
  const { colorFn } = useColorFunction();
  const colorSeries = useMemo(() => {
    try {
      const colors = colorFn?.series(rootColor, tones, step / 100);
      return (colors ?? Array.from({ length: 9 }, () => rootColor)).reverse();
    } catch (e) {
      console.error('[Generate Color Series]', e);
    }
    return Array.from({ length: 9 }, () => rootColor);
  }, [rootColor, tones, step]);

  return (
    <div
      className={styles.color_column}
      style={{ transform: `rotate(${rotate}deg) translateX(-50%)` }}>
      {colorSeries.map((c, index) => (
        <ColorBlock
          key={`${c}-${index}`}
          dimmed={!actived}
          color={c}
          isRoot={isEqual(c, rootColor) && isEqual(rotate, 0)}
        />
      ))}
    </div>
  );
}
